/*
 *   Functions to deal with uncertainty of the laser data
 */


/**** 
  Transform of the uncertainty of an entire image 
*****/

void impt_inc_xyro(igla *ig);
    

/**************
  Transform for a single point
***************/

void point_inc_xyro(p2d *pt);



void point_inc_roxy(p2d *pt);




 





/*
 *  Kalman's estimate of the image igl.
 *
 *  The grouping itself has been done by segmentation() in lasegm.c which
 *  eventualy calls this function at the very end.
 *  The list of groups is in fact a list of segments. The fields
 *  sg->iend and sg->ifirst are the indices of the limits of each group in the
 *  array igl->tbp
 */

void las_segm_unc(igla *igl);




/********
  computes the uncertain intersection + its variance matrix
  between 2 uncertain lines 
********/

int intersg_unc(Sgl *sg1,Sgl *sg2,p2d *ptp);
  



/********
  computes the uncertainty of the intersection
  between 2 uncertain lines; the intersection
  itself is supposed already calculated.
********/

int intersg2_unc(Sgl *sg1,Sgl *sg2,p2d *ptp);
    



/*******
  computes the uncertainty of a vertex;
  vertex = last point of sglp = first point of sglc
  The vertex itself is assumed already computed.
*******/


void EndSS_inc (Sgl *sglp,Sgl *sglc);




/*********
  initialize a line with its variance from 
  p1 and p2 with their variance.

  Cartesian coordinates version.

  Convention: ->o is the angle (radian) of the opposite normal 
              ->d is the orthogonal distance to the origin
	      ->vx = -cos(o)
	      ->vy = -sin(o)
	      vx,vy, is the normal going to the right when you walk from 
	      p1 to p2

  Looks better than the same thing in polar coodinates.
  Remains to be explained ....
***********/
     

int initdt2 (p2d *p1,p2d *p2,Sgl *dt);





/* Kalman's Observation Functions */

/* line equation in polar coordinates 
   
   ro(pt)*cos(phi(pt) - phi(sg)) - dist2origin(sg) = 0

*/

void dist(double *pt,double *sg,double *d);


/* Jacobian  around pt */

void ddistp(double *pt,double *sg,double *dp);


/* Jacobian around sg */

void ddistsg(double *pt,double *sg,double *dsg);





/********
  special identity functions when the x[1] is an angle
********/

void idta(double *z,double *x,double *f);


void didtax(double *z,double *x,double *r);

	  



void didtaz(double *z,double *x,double *r);



/*************
  x[2] is an angle 
*************/

void idta3(double *z,double *x,double *f);


void didta3x(double *z,double *x,double *f);



void didta3z(double *z,double *x,double *f);














