#ifndef _lasdef
#define _lasdef


#include "dbliste.h"

/************************************************************************

  declaration of Laser data structures  

-----------------------------------------------------------------------*/

/* type P2D ( laser point) */
/**************************/

typedef struct pt2d {
  double x,y; /* cartesian representation */
  double vx,vxy,vy; /* variance on the cartesian representation */
  double r,o; /* polar representation */
  double vr,vro,vo;/* variance on the polar representation */
  double co,so; /* cosine and sine of o */
  } p2d;


/********
  Sgl : laser segment

  Conventions:
      when the segment has been perceived through the sensus 500 
      and in general if you want  to be consistent with this representation, 
      the free space is on the left side when you go from the first point to
      the last point;
      
      vx,vy is the normal to the segment and goes to the left (free space)

      ->o is the angle of the normal = atan2(->vy,->vx)
      

*********/

typedef struct dt2d {
  double vx,vy,d; /* inifinite line parameters x.vx + y.vy + d = 0 */
  double r,o,vr,vro,vo;/* polar representation and its variance */
  long ifirst,ilast; /* indices ofthe first and last points in  
                       the array of point from which the segment has
                        been computed 
                        */
  char typf,typl; /* extremity point types ( V for vertex or B for  
                                             border)  */
  struct pt2d first,last; /* extremity points */
  struct dt2d *sfirst, *slast; /* segment possibly connected 
                                (sfirst != NULL <=> typf == 'V')*/
  double er; /* mse on  d */
  char flg; /* user flag ...*/ 
} Sgl;

/********
 Imgl = laser image = laser point array + list segments (if any) 
                      computed from the array + robot and turret attitude
                      when the raw data has been acquired 
*********/

typedef struct imgl {
  int nbp; /* number of points */
  p2d *tbp; /* array of points */
  long nsg; /* number of segments */
  PDBL *lsgl; /* list of segments */
  double x,y; /* position of the robot when the perception had been made */
  double cs,sn; /* cosine and sine of the steering angle */
  double cst,snt; /* same thing for the turret */
  double var[6] ; /*** variance on the position *****/
  double enc[4]; /* encoder position at the time of the perception 
		  (inches and radians).
		 NOTE : this is not thesame as the robot position which is relative to
		 an absolute position in the environnment. 
		 This is relative to the starting point of the robot.
		 */
  char coosys; /* says in which system of coordinates the image is represented;
		  A for Absolute
		  S for steering angle + center of the robot
		  T for turret angle + center of the robot 
		*/
  char *usr; /* pointer to a user's field */
  
} igla;
/*******
 inertia "matrix" for least square estimation
********/

struct matiner {
  double sx,sy,sx2,sy2,sxy;
};



#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif





/*******
Macros for List
*******/

#define Sgel(l) ((Sgl *)l->el)
#define Ptel(l) ((p2d *)l->el)
#define Igel(l) ((igla *)l->el)


#endif




