/*
 *         Laser Data Segmentation
 *
 * two versions:
 *  segmentation(igl,option)
 * 
 *      input: an image of type "*igla"
 *          This image contain an array of igl->nbp
 *          raw points :  igl->tbp
 *      output: the same image with the list igl->lsgl of
 *           segment computed from the points igl->tbp.
 *
 *      option: 0 for least square estimation
 *           1 for kalman estimation
 *   
 *  get_sg_host(sgt,nbp,laser_val)
 *       
 *      input: the number of point "nbp" in the 
 *             laser pixel array "laser_val"
 *        
 *      output: the array of segment "sgt"
 *              and the number of segment (returned value)
 *      
 *
 *    Points are assumed coming from the same central position and
 *    ordered in angle in a trigonometric manner.
 *    
 *
 *   function conf_sg_host:
 *                         sets the parameters:
 *
 *                           thres (on the least square error)
 *                           nbmin (minimum number of points in
 *                                  a segment)
 *                           gap (maximum space between two consecutive
 *                                points in a segment)
 *
 *                           laser_range : maximum range of the laser system
 *   function segmentation: computes the longest segments from
 *                          igl->tbp verifying the constraints set
 *                          by Set_segm_param
 *

 *   function first_discont_point       : returns TRUE if there is a discontinuity
 *                           between the  points j and k. If TRUE,
 *                           m is the discontinuous point.
 *   function first_continuous_point      : returns the index of the next
 *                              point, after k, which could be the
 *                              first point of a segment.
 *
 *   functions EndSS, eEndBP, EndBB :
 *                           computes the different kind of
 *                           intersections
 */








/***************
 * FUNCTION:     conf_sg_host
 * PURPOSE:      set parameters for the segmentation
 * ARGUMENTS:    double thr : threshold on the mean square error
                 
		 defaults to 0.009.

		 If a segment has a higher mean error value
		 that thres, it will be rejected.
 *               int nb : minimum number of points in a segment
                          
 *               int max_gap : maximum distance between two points
                               in a segment.
 *               laser_range : maximum range of the laser system
 * ALGORITHM:    
 * RETURN:       void 
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/

int conf_sg_host(double thr,int nb,double max_gap, double laser_range);



/***************
 * FUNCTION:     segmentation
 * PURPOSE:      
 * ARGUMENTS:    igla *ig : ig = pointer on the image containing
                            the array of point (ig->tbp).
		 int op : option for kalman filtering: 
                                0 = least square only
				1 = grouping using
				    least square +
				    estimation using
				    Extended Kalman Filtering

 * ALGORITHM:    least square regression assuming points are
                 ordered in angle around 0,0 
		 If op = 1, at the very end, kalman estimation
		 using the grouping produced by the least square
		 algorithm.
		 returns always 0 (??)
 * RETURN:       int 
 * SIDE EFFECT:  
 * CALLS:        las_segm_unc()/las_unc.c, first_continuous_point(),first_discont_point(),
                 alined(),stepping_back()  
 * CALLED BY:    
 ***************/


int segmentation (igla *ig,int op);

