/****************************
Little (but useful !) functions 
dealing with points and segments

NOTE: p2d and Sgl structures are described in lasdef.h
****************************/





/***************
 * FUNCTION:     norm_ang
 * PURPOSE:      normalizes an angle between p and 2*Pi + p
 * ARGUMENTS:    double a :
 *               double p :
 * ALGORITHM:    
 * RETURN:       double : an angle between p and 2*M_PI + p
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/

double norm_ang(double a,double p);
   


/***************
 * FUNCTION:     intersg
 * PURPOSE:      computes the intersection between 
                 s1 and s2. Puts the result in p.
                 Returns 0 if segments are parallel
                 Otherwise, returns 1
 * ARGUMENTS:    Sgl *s1 :
 *               Sgl *s2 :
 *               p2d *pt :
 * ALGORITHM:    
 * RETURN:       int 
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/

int intersg(Sgl *s1,Sgl *s2,p2d *pt);
    



/***************
 * FUNCTION:     proj_orth
 * PURPOSE:      computes the orthogonal projection 
                 of pt on dt.
                 Result in ptp (which has to be 
		 allocated before the call)
 * ARGUMENTS:    Sgl *dt :
 *               p2d *pt :
 *               p2d *ptp :
 * ALGORITHM:    
 * RETURN:       void 
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/


void proj_orth(Sgl *dt,p2d *pt,p2d *ptp);


/***************
 * FUNCTION:     dt2pt
 * PURPOSE:      Initialize a line dt
                 from 2 points pt1,pt2
                 (computes dt->vx,dt->vy,dt->d)
                 vx,vy is the normal with the convention that 
		 it goes to the left when one walk along from pt1 to pt2.
		 d = -x*vx -y*vy where (x,y) is any point belonging
		 to dt. 
		 In particular:
		 d = -pt1->x*dt->vx - pt1->y*dt->vy

 * ARGUMENTS:    p2d *pt1 :
 *               p2d *pt2 :
 *               Sgl *dt :
 * ALGORITHM:    
 * RETURN:       void 
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/

void dt2pt (p2d *pt1,p2d *pt2,Sgl *dt);
   




/***************
 * FUNCTION:     sg_calc_param
 * PURPOSE:      computes vx,vy,d from extremity points
                 such that ||vx^2 + vy^2|| = 1 and the vector 
                 vx,vy goes on the left side of sg when going from
                 the begining to the end;
		 Basically the same as dt2pt but assuming 
		 pt1 and pt2 are already in sg->first and sg->last
		 respectively.
		 Computes also the polar representation ->r and ->o
		 of sg.
 * ARGUMENTS:    Sgl *sg :
 * ALGORITHM:    
 * RETURN:       void 
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/

void sg_calc_param(Sgl *sg);
     



/***************
 * FUNCTION:     InsidePoly
 * PURPOSE:      tests if the point defined by
                 x = pos[0], y = pos[1] is inside
		 the polygone define by the segment list
		 poly->lsgl.
		 Returns 1 if inside
		         0 if outside
                         -1 if poly = NULL or
			    if the list of segments is not
			    closed.
			 
 * ARGUMENTS:    double *pos :
 *               igla *poly :
 * ALGORITHM:    
 * RETURN:       int 
 * SIDE EFFECT:  
 * CALLS:        
 * CALLED BY:    
 ***************/

int InsidePoly(double *pos,igla *poly);
  
  




